import ucanlintools.LUC
import threading, time, sys, os
from ucanlintools.LDF_parser import parseLDF
import binascii

rx_count = 0

def rx_any(f):
        global rx_count
        rx_count += 1 

def rx_new_data(f):
        global message
        print (str(rx_count) + " " + hex(f.id) + ": " +(f.data.hex()))
        if (message.id == f.id):
                message.decode(f.data)
                print(message.diff_str())

ldf = parseLDF("D:\\ldfe.ldf")        
message = ldf.get_message_by_name('FRAME_A')


lin = ucanlintools.LUC('COM7')

lin.set_frame_rx_handler(rx_any)
lin.set_new_frame_rx_handler(rx_new_data)

lin.openAsMaster() 

lin.addReceptionFrameToTable(52,8)
lin.addReceptionFrameToTable(54,8)
lin.enable()

input("Press Enter to continue... \r")

lin.disable()
lin.close()
lin.deInitSerial()     

os._exit(1)    
        


import bitstruct

class LINFrame:
    def __init__(self,frame_id, frame_len, name, signals, publisher):
        self.id = frame_id
        self.len = frame_len
        self.name = name
        self.signals = signals
        bitsting = ""
        for s in self.signals:
            s['value'] = s['default_val']
            bitsting += "u" + str(s['size_bits'])
        self.publisher = publisher
        self.bitsting = bitsting
        self.__cf = bitstruct.compile(bitsting)        
        self.__last_values = {}

    def encode(self, lin_data):
        for dd in lin_data:
            item = next((x for x in self.signals if x['name'] == dd), None)
            if item != None:
                item['value'][0] = lin_data[dd]
        
        data = list(map(lambda s: s['value'][0], self.signals))        
        p = self.__cf.pack(*data)
        return p

    def decode(self, lin_data):
        p = self.__cf.unpack(lin_data)  
        for i,s in enumerate(self.signals):                                    
            s['value'][0] = p[i]
        return p
    def diff_str(self):
        outStr = "{"
        for i,s in enumerate(self.signals):
            val = s['value'][0]
            if (len(self.__last_values) <= i) or (val != self.__last_values[i]):
                self.__last_values[i] = val
                outStr += '\'' + s['name']  + '\'' + " : " + str(val) + ','
        if len(outStr) > 1:
            outStr = outStr[:-1] + "}"
        else:
            outStr += "}"
        return outStr   





import ucanlintools.LUC
import ucanlintools.LDF_parser
from ucanlintools.LDF_parser import parseLDF


from pynput import keyboard
import time

""" LIN Master simulation, 2 buttons slave + backlight setup. + LDF """

ldf = parseLDF("example_LDF.ldf")
master_frame = ldf.get_message_by_name('Backlight')
slave_frame = ldf.get_message_by_name('Buttons_Status')

BacklightLevel = 1

def rx_new_data(f):
    global slave_frame
    if (f.id == slave_frame.id):
        slave_frame.decode(f.data)
        print ("LIN> " + slave_frame.diff_str())

lin = ucanlintools.LUC('COM8')

lin.set_new_frame_rx_handler(rx_new_data)

lin.openAsMaster() 

lin.addReceptionFrameToTable(slave_frame.id,slave_frame.len)
lin.addTransmitFrameToTable(master_frame.id,b'00')

lin.enable()

def DataToFrame():
    global BacklightLevel
    aa = master_frame.encode({'backlight_level': BacklightLevel})
    lin.addTransmitFrameToTable(master_frame.id,aa)
    print(aa)

def on_release(key):
    global BacklightLevel
    try:
        if key.char == 'e':
            if (BacklightLevel < 0x0f):
                BacklightLevel = BacklightLevel + 1
                DataToFrame() 
        if key.char == 'd':
            if (BacklightLevel > 0):
                BacklightLevel = BacklightLevel - 1
                DataToFrame()
        
    except AttributeError:
        if key == keyboard.Key.esc:
            # Stop listener
            return False
        # print("special key")


with keyboard.Listener(
        on_release=on_release) as listener:
    listener.join()
    print("SIM END")
    lin.disable()
    lin.close()
    lin.deInitSerial()     





from lark import Lark, Transformer
from ucanlintools.LINFrame import *

def ParseIntOrHex(x):
    try:
        return (int(x))
    except ValueError:
        return (int(x,16)) 



class LDF:
    """LDF parsing class.    
    
    Parameters
        ----------
        nodes : lin nodes
            The name of the animal
        frames : lin frames with id and signals description
        
    """

    def __init__(self,nodes, frames, signals):
        self.nodes = nodes
        lin_frames  = []
        for f in frames:
            for fs in f['frame_signals']:
                for s in signals:
                    if (fs['name'] == s['signal_name']):
                        fs.update(s)
                        del fs['signal_name']
            lin_frames.append(LINFrame(f['frame_id'], f['frame_len'], f['frame_name'], f['frame_signals'], f['publisher'] ) )                        
        self.frames = lin_frames
        
    
    def get_message_by_name(self, name):
        """Get Message by message name
        
        Returns
        ----------
        LINFrame object
        """
        for f in self.frames:
            if (f.name == name):
                return f
        return
    def get_message_by_id(self, fid):
        """Get Message by message id
        
        Returns
        ----------
        LINFrame object
        """
        for f in self.frames:
            if (f.id == fid):
                return f
        return


class TreeToJson(Transformer):
    def ldf_node_name(self,s):
        return s[0][0:]
    def signal_name(self, s):
        return s[0][0:]
    def signal_size(self, s):
        return ParseIntOrHex(s[0])
    def signal_bit_offset(self, s):
        return ParseIntOrHex(s[0])
    def ldf_node_master(self,s):
        return s[0][0:]
    def ldf_node_slaves(self,s):
        return s[0][0:]

    def ldf_nodes(self,s):
        return {'nodes':s}
    def ldf_signals(self,s):
        return {'signals': s}
    def ldf_frames(self,s):
        return {'frames': s}
   
    # ldf_container = dict
    def ldf_container(self,s):
        return s

    def ldf_signal(self,s):
        return {'signal_name':s[0], 'size_bits':s[1], 'default_val': s[2], 'publisher': s[3], 'subscriber': s[4]}
    def ldf_frame(self,s):
        return {'frame_name':s[0], 'frame_id':s[1], 'publisher':s[2], 'frame_len':s[3], 'frame_signals':s[4:]}
    def ldf_frame_signal(self,s):
        return {'name':s[0],'bit_offset':s[1]}
    def ldf_frame_name(self,s):
        return s[0][0:] 
    def ldf_frame_id(self,s):
        return ParseIntOrHex(s[0])
    def ldf_frame_len(self,s):
        return ParseIntOrHex(s[0])
    def ldf_signal_size(self,s):
        return ParseIntOrHex(s[0])
    def ldf_signal_bit_offset(self,s):
        return ParseIntOrHex(s[0])
    def ldf_signal_name(self,s):
        return s[0][0:]

    def ldf_signal_default_value(self,s):
        s = s[0]
        s = s.replace('{','').replace('}','').split(',')
        o = []
        for x in s:
            o.append(ParseIntOrHex(x))
        return o

    # start = dict 
    def start(self,s):
        return s[0]

    def ldf_node_atributes(self,s):
        return
        # return {"ldf_node_atributes" : "NOT_IMPLEMENTED"}
    def ldf_node_atributes_node(self,s):
        return
        # return {"ldf_node_atributes_node" : "NOT_IMPLEMENTED"}

    def ldf_schedule_table(self,s):
        return
        # return {"ldf_schedule_table" : "NOT_IMPLEMENTED"}
    def ldf_signal_representation(self,s):
        return
        # return {"ldf_signal_representation" : "NOT_IMPLEMENTED"}
    def ldf_signal_representation_node(self,s):
        return
        # return {"ldf_signal_representation_node" : "NOT_IMPLEMENTED"}
    def ldf_signal_encoding_types(self,s):
        return
        # return {"ldf_signal_encoding_types" : "NOT_IMPLEMENTED"}
    def ldf_diagnostic(self,s):
        return
        # return {"ldf_diagnostic" : "NOT_IMPLEMENTED"}
    def ldf_diagnostic_frames(self,s):
        return
        # return {"ldf_diagnostic_frames" : "NOT_IMPLEMENTED"}
    def ldf_header(self,s):
        return
        # return {"ldf_header" : "NOT_IMPLEMENTED"}


def parseLDF(file):
    """Parse LDF file
        
    Returns
    ----------
    LDF object
    """
    # json_parser = Lark(open("..\ucanlintools\ldf.lark"),parser="lalr")
    json_parser = Lark(''' 
    start: ldf_container

?ldf_container: ldf_header [(ldf_nodes | ldf_signals | ldf_frames | ldf_diagnostic | ldf_diagnostic_frames | ldf_node_atributes | ldf_schedule_table | ldf_signal_encoding_types | ldf_signal_representation)*]

?ldf_nodes : "Nodes" "{" [ldf_node_master] [ldf_node_slaves] "}" 
?ldf_signals : "Signals" "{" [ldf_signal (ldf_signal)*] "}"  
?ldf_frames : "Frames" "{" [ldf_frame (ldf_frame)*] "}" 

?ldf_diagnostic : "Diagnostic_signals" "{" [(ANY_SEMICOLON_TERMINATED_LINE)*] "}"
?ldf_diagnostic_frames : "Diagnostic_frames" "{" [(ldf_diagnostic_master_req | ldf_diagnostic_slave_resp)*] "}"

?ldf_diagnostic_master_req: "MasterReq:" [C_INT] "{" [(ANY_SEMICOLON_TERMINATED_LINE)*] "}"
?ldf_diagnostic_slave_resp: "SlaveResp:" [C_INT] "{" [(ANY_SEMICOLON_TERMINATED_LINE)*] "}"

?ldf_node_atributes: "Node_attributes" "{" ldf_node_atributes_node? "}"
?ldf_node_atributes_node: ANY_OPENED_BLOCK [ANY_SEMICOLON_TERMINATED_LINE*] ANY_OPENED_BLOCK [ANY_SEMICOLON_TERMINATED_LINE*] "}" "}"

?ldf_schedule_table: "Schedule_tables" "{" [ldf_schedule_table_node*] "}" 
?ldf_schedule_table_node: ANY_OPENED_BLOCK [ANY_SEMICOLON_TERMINATED_LINE*] "}"

?ldf_signal_encoding_types: "Signal_encoding_types" "{" [ldf_signal_encoding_types_node*] "}" 
?ldf_signal_encoding_types_node: ANY_OPENED_BLOCK [(ANY_SEMICOLON_TERMINATED_LINE*)] ["}"] [ANY_CLOSED_BLOCK]

?ldf_signal_representation: "Signal_representation" "{" [ldf_signal_representation_node*] "}" 
?ldf_signal_representation_node: [(ANY_COLON_TERMINATED_LINE | ANY_SEMICOLON_TERMINATED_LINE)*]

?ldf_node_master: "Master:" ldf_node_name "," /.*;/  
?ldf_node_slaves: "Slaves:" ldf_node_name? ";" 

ldf_node : ldf_signal_name  ":" ldf_node_name ";" 
ldf_signal :  ldf_signal_name ":" ldf_signal_size "," ldf_signal_default_value "," ldf_node_name "," ldf_node_name ";" 
ldf_frame : ldf_frame_name  ":" ldf_frame_id "," ldf_node_name "," ldf_frame_len ["{" (ldf_frame_signal)* "}"] 


ldf_frame_signal: ldf_signal_name "," ldf_signal_bit_offset ";" 
ldf_frame_name: CNAME 
ldf_frame_id : C_INT 
ldf_frame_len : C_INT 

ldf_signal_default_value: C_INITIALIZER_LIST|C_INT 
ldf_signal_size: C_INT 
ldf_signal_bit_offset: C_INT 
ldf_signal_name: CNAME 

ldf_node_name: CNAME 

ANY_OPENED_BLOCK: /.*{/
ANY_CLOSED_BLOCK: /.*}/

ANY_SEMICOLON_TERMINATED_LINE: /.*;/
ANY_COLON_TERMINATED_LINE: /.*,/



C_INITIALIZER_LIST: ("{"|"{ ") C_INT ([","|", "]C_INT)* ("}"|" }")
C_INT: ("0x"HEXDIGIT+) | ("-"? HEXDIGIT+) 

ldf_header: (ldf_header_lin | ldf_header_channel)*  
ldf_header_lin : "LIN_"/.*;/
ldf_header_channel : "Channel_name"/.*;/

%import common._STRING_INNER

%import common.HEXDIGIT 
%import common.INT
%import common.WORD
%import common.CNAME
%import common.ESCAPED_STRING
%import common.SIGNED_NUMBER
%import common.WS
%import common.WS_INLINE
%ignore WS
%ignore WS_INLINE
    
    
    
    ''',parser="lalr")

    f=open(file, "r").read()
    tree = json_parser.parse(f)
    # print(tree.pretty())
    
    json_data = TreeToJson().transform(tree)
    ldf = {}
    for a in json_data:
        if (a != None):
            ldf.update(a)

    out_ldf = LDF(ldf['nodes'],ldf['frames'],ldf['signals']) 
    

    return out_ldf



import unittest
from pprint import pprint
from ucanlintools.LDF_parser import parseLDF

class LDFParserTests(unittest.TestCase):
        # def setUp(self):

        # def tearDown(self):

        def test_ldf(self):
            ldf = parseLDF("D:\\e.ldf")
            self.assertEqual(ldf.frames[0].id, 52)
        
        def test_ldf_with_spaces(self):
            ldf = parseLDF("D:\\ldfe.ldf")        
            message = ldf.get_message_by_name('FRAME_A')

            self.assertEqual(ldf.frames[0].id, 52)

        def test_encode_decode_message(self):
            ldf = parseLDF("D:\\ldfe.ldf")        
            message = ldf.get_message_by_id(52)

            bbb = b'\x00\x00\x00\x00\x00\x00' 
            message.decode(bbb)                        
            diff_str = message.diff_str()

            aaa = message.encode({'SIGNAL_A': 1})            
            diff_str = message.diff_str()
            message.decode(aaa)

            message.decode(bbb)                        
            diff_str = message.diff_str()
            self.assertEqual(diff_str, "{'SIGNAL_A' : 0}")

def main():
    unittest.main()

if __name__ == "__main__":
    main()


import serial
import math
import time
import threading
import logging

logger = logging.getLogger()


class LINFrame:
    """LIN Frame class.
        

    Attributes
    ----------
    id: int 
        frame id
    data: bytearray
        frame data as byte array
    """
    id = 0
    data = bytearray()


class LUC:

    def __def_frame_rx_handler(self,f):
        pass

    def __def_new_frame_rx_handler(self,f):
        pass

    def __init__(self, com):
        self.lastframes = {}
        self.currentframes = {}
        self.ser = serial.Serial(com,timeout=0.02)
        if (self.ser == None):
            raise NameError('SerialPortNotPresent') 
        self.ser.reset_output_buffer()
        self.ser.reset_input_buffer()
        
        self.__rx_byte_thread_handler = threading.Thread(target=self.__rxThread, args=())                     
        self.__rx_frame_thread_handler = threading.Thread(target=self.__newFrameProcess, args=())
        self.__uart_buffer = ''
        self._stop_event = threading.Event()        
        self.frame_rx_handler = self.__def_frame_rx_handler
        self.new_frame_rx_handler = self.__def_new_frame_rx_handler

    def set_frame_rx_handler(self, rx_handler):
        """Sets handler for frame reception, will be called after any frame reception.
        
        Parameters
        ----------  
            rx_handler : function handler, with LINFrame argument 
        """

        self.frame_rx_handler = rx_handler

    def set_new_frame_rx_handler(self, rx_handler):
        """Sets handler for frame reception, will be called only when there is diffrent data on frame received.

        Parameters
        ----------  
            rx_handler : function handler, with LINFrame argument 
        
        """
        self.new_frame_rx_handler = rx_handler

    def flushData(self, data):
        self.ser.write(data)
        logger.info(data)
    

    def reqestFirmwareVersion(self):
        self.flushData(b'v\r')
        line = self.ser.readline().decode("utf-8")         
        return line.replace("v","").replace("\r","")

    def openAsMaster(self):
        """Open LIN Converter in Master mode"""
        self.flushData(b'O\r')        
        return (self.ser.readline().decode("utf-8") == '\r')

    def lowSpeed(self):
        """LIN low speed"""
        self.flushData(b'S2\r')        
        return (self.ser.readline().decode("utf-8") == '\r')
    
    def highSpeed(self):
        """LIN high speed"""
        self.flushData(b'S1\r')        
        return (self.ser.readline().decode("utf-8") == '\r')

    def openAsSlave(self):
        """Open LIN Converter in slave mode"""
        self.flushData(b'L\r')        
        return (self.ser.readline().decode("utf-8") == '\r')

    def openAsMonitor(self):
        """Open LIN Converter in monitor mode"""
        self.flushData(b'l\r')        
        return (self.ser.readline().decode("utf-8") == '\r')

    def close(self):
        """Close LIN connection"""
        self.flushData(b'C\r')        
        return (self.ser.readline().decode("utf-8") == '\r')

    def addTransmitFrameToTable(self,id, data):
        """Add new frame to LIN Converter table"""

        if (type(data)) is int:
            data_str = hex(data).replace("0x","")
        else:
            data_str = data.hex()
        data_len = math.floor(len(data_str)/2)
        line = ('t0' + hex(id).replace("0x","").rjust(2,'0') + str(data_len) + data_str + '\r') 
        self.flushData(line.encode())
        return (self.ser.readline().decode("utf-8") == 'z\r')

    def addReceptionFrameToTable(self,id, len):
        """Add new header to LIN Converter table"""
        line = ('r0' + hex(id).replace("0x","").rjust(2,'0') + str(len) + '\r') 
        self.flushData(line.encode())
        return (self.ser.readline().decode("utf-8") == 'z\r')

    def enable(self):
        """Enable frame sending"""
        self.flushData(b'r1ff0\r')
        # r = (self.ser.readline().decode("utf-8") == 'z\r')
        self.__rx_byte_thread_handler.start()
        self.__rx_frame_thread_handler.start()
        return 1

    def disable(self):
        """Stop reception"""
        if (self.__rx_byte_thread_handler.is_alive()):
            self._stop_event.set()
            self.__rx_byte_thread_handler.join()
            self.__rx_frame_thread_handler.join()
        
        time.sleep(0.010)

        self.flushData(b'r2ff0\r')
        return (self.ser.readline().decode("utf-8") == 'z\r')

    def __isStartingChar(self,c):
        return ((c == 't') or (c == 'r'))

    def __rxThread(self):
        while True:
            if (self._stop_event.is_set()):
                break
                
            while(self.ser.in_waiting > 0):                
                cc = self.ser.read(self.ser.in_waiting).decode('ascii')
                self.__uart_buffer += (cc)                                   
            time.sleep(0.001)
    
    def __newFrameProcess(self):
        while True:
            if (self._stop_event.is_set()):
                break
            f = self.__waitForFrame()
            if (f != None):
                self.frame_rx_handler(f)
                if(self.currentframes.get(f.id) == None) or (self.currentframes[f.id] != f.data):                
                    self.currentframes[f.id] = f.data
                    self.new_frame_rx_handler(f)

    def __waitForFrame(self):
        while True:
            if (self._stop_event.is_set()):
                return None

            if (len(self.__uart_buffer) > 1000):
                self.__uart_buffer = ''

            start = self.__uart_buffer.find('t')
            end =  self.__uart_buffer.find('\r')

            if (start != -1 and end != -1):
                if (start > (end - 5)):
                    self.__uart_buffer = self.__uart_buffer[start:]
                else: 
                    frame_string = self.__uart_buffer[start:end]

                    self.__uart_buffer = ''

                    lf = LINFrame()                
                    lf.id = int(frame_string[2:4],16)
                    lf.data = bytes.fromhex(frame_string[5:])       
                    return lf
            
            time.sleep(0.001)

    def deInitSerial(self):
        self.ser.flush() 
        self.ser.close()
        del self.ser

    def __del__(self):
        self.disable()
        self.deInitSerial()
        


import ucanlintools.LUC
import ucanlintools.LDF_parser

from pynput import keyboard
import time

""" LIN Slave simulation, 2 buttons slave + backlight setup. """

key1 = 0
key2 = 0
BacklightLevel = 0

def rx_new_data(f):
    global key1
    global key2
    global BacklightLevel
    # print ("LIN<" + f.data.hex())
    if (f.id == 11):
        bckl = int.from_bytes(f.data, byteorder='little')
        print ("new backlight level " + str(bckl))
        BacklightLevel = f.data[0]
        KeysToFrame(key1,key2)

lin = ucanlintools.LUC('COM7')

lin.set_new_frame_rx_handler(rx_new_data)

lin.openAsSlave() 

lin.addReceptionFrameToTable(11,1)
lin.addTransmitFrameToTable(12,b'0000')

lin.enable()

def KeysToFrame(key1, key2):
    global BacklightLevel
    aa = 0 
    if (key1):
        aa = 1
    if (key2):
        aa += 16
    aa += (BacklightLevel * 256)
    aa = aa.to_bytes(2, byteorder='little')
    # print (aa)
    lin.addTransmitFrameToTable(12,aa)

def on_press(key):
    global key1
    global key2
    try:
        # print('key {0} pressed'.format(key.char))
        if key.char == 'q':
            key1 = 1
            KeysToFrame(key1,key2)
        if key.char == 'w':
            key2 = 1
            KeysToFrame(key1,key2)
    except AttributeError:
        pass

def on_release(key):
    global key1
    global key2

    try:
        # print('{0} released'.format(key))
        if key.char == 'q':
            key1 = 0
            KeysToFrame(key1,key2)
        if key.char == 'w':
            key2 = 0
            KeysToFrame(key1,key2)
        
    except AttributeError:
        if key == keyboard.Key.esc:
            return False


with keyboard.Listener(
        on_press=on_press,
        on_release=on_release) as listener:
    listener.join()
    print("SIM END")
    lin.disable()
    lin.close()
    lin.deInitSerial()     





import unittest
import time
from ucanlintools.LUC import LUC

ff = 0
count = 0

def rx_any(f):
    global ff
    global count
    
    count = count + 1
    ff = f

class LUCTesting(unittest.TestCase):
    def setUp(self):
        self.master = LUC('COM11')
        self.slave = LUC('COM7')

        ll = self.master.close()
        ll = self.slave.close()

    def tearDown(self):
        del self.master
        del self.slave

    def test_firmware_version(self):        
        ll = self.master.reqestFirmwareVersion()
        self.assertEqual(ll, '0103')
        ll = self.slave.reqestFirmwareVersion()
        self.assertEqual(ll, '0103')

    def test_comunication(self):
        global ff
        global count

        ll = self.master.lowSpeed()
        ll = self.slave.lowSpeed()

        ll = self.master.openAsMaster() 
        ll = self.slave.openAsSlave()       
        self.assertEqual(ll, 1)
        
        ll = self.master.disable()
        ll = self.slave.disable()
        self.assertEqual(ll, 1)        
        
        ll = self.master.addTransmitFrameToTable(0x1,0x2233)
        ll = self.slave.addReceptionFrameToTable(0x1,0x2)
        self.assertEqual(ll, 1)                
        
        ll = self.slave.enable()
        ll = self.master.enable()        
        self.assertEqual(ll, 1)

        self.slave.set_frame_rx_handler(rx_any)
        
        time.sleep(1)
        
        self.slave.disable()
        self.master.disable()

        print(count)
        self.assertGreater(count, 40)
        self.assertEqual(ff.id, 1)
        self.assertEqual(int.from_bytes(ff.data,byteorder='big'), 0x2233)

def main():
    unittest.main()

if __name__ == "__main__":
    main()


# python setup.py sdist
# python -m twine upload dist/*
 
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
     name='ucanlintools',  
     version='0.0.8',
     author="ucan",
     author_email="ucandevices@mail.com",
     description="Local Interconnect Network (LIN) support for python",
     long_description=long_description,
     long_description_content_type="text/markdown",
     url="https://github.com/uCAN-LIN/LinUSBConverter",
     packages = ['ucanlintools'],
     package_data = { '': ['*.lark'] },
     license = 'MIT',
     keywords = ['LIN', 'LDF', 'LUC', 'uCAN', 'parser'],
     install_requires = ['pyserial','lark', 'bitstruct'],

     py_modules=['LUC','LDF_parser', 'LINFrame'],
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
     project_urls={
        "Documentation": "https://ucandevices.github.io/",
        "Source Code": "https://github.com/uCAN-LIN/LinUSBConverter/tree/master/python_lib",
     }
 )